#!/usr/bin/apt-get script
-- This script will install NVIDIA driver
-- Author: Sergey V Turchin <zerg@altlinux.org>

function os.getexecout(cmd)
  local f = assert(io.popen(cmd, 'r'))
  local s = assert(f:read('*a'))
  f:close()
  s = string.gsub(s, '^%s+', '')
  s = string.gsub(s, '%s+$', '')
  s = string.gsub(s, '[\n\r]+', ' ')
  return s
end

if script_slot == "Scripts::AptGet::Help::Command" then
    print(_("   install-nvidia - Install NVIDIA driver"))
    return
end

if command_args and command_args[1] ~= "install-nvidia" then return end
command_consume = 1

kernel_pkg_nameversion = os.getexecout("rpmquery -f --qf '%{NAME}#%{EPOCH}:%{VERSION}-%{RELEASE}' /boot/vmlinuz-`uname -r`")
pkg_kernel = pkgfind(kernel_pkg_nameversion)
if pkg_kernel == nil then
    kernel_pkg_nameversion = os.getexecout("rpmquery -f --qf '%{NAME}' /boot/vmlinuz-`uname -r`")
    pkg_kernel = pkgfind(kernel_pkg_nameversion)
    if pkg_kernel == nil then
	apterror(_("Package not found for your current kernel."))
	return
    end
end
-- print("Kernel " .. kernel_pkg_nameversion)
pkg_kernel_name = pkgname(pkg_kernel)
pkg_kernel_ver = pkgvercur(pkg_kernel)

pkg_kernel_prov_name = {}
pkg_kernel_prov_verstr = {}
kernel_provlist = verprovlist(pkg_kernel_ver)
for i, kprov in ipairs(kernel_provlist) do
    if string.find(kprov.name, "kernel-image-", 0, true) == 1 then
	pkg_kernel_prov_name = kprov.name
	pkg_kernel_prov_verstr = kprov.verstr
	-- print("Kernel provide " .. kprov.name .. "#" .. kprov.verstr)
    end
end

-- Find kernel-modules-nvidia package
nvidia_pkg = nil
nvidia_ver = nil
for i, pkg in ipairs(pkglist()) do
    if string.find(pkgname(pkg), "kernel-modules-nvidia-", 0, true) == 1 then
	-- print("Package " .. pkgname(pkg))
	ver = pkgvercand(pkg)
	if ver == nil then
	    ver = pkgvercur(pkg)
	end
	if ver ~= nil then
	    for i, dep in ipairs(verdeplist(ver)) do
		if dep.name == pkg_kernel_prov_name and dep.verstr == pkg_kernel_prov_verstr then
		    if( nvidia_pkg == nil or nvidia_ver == nil or verstrcmp(verstr(ver),verstr(nvidia_ver)) > 0 ) then
			-- print("Package " .. pkgname(pkg) .. " require " .. dep.name .. "#" .. dep.verstr)
			nvidia_pkg = pkg
			nvidia_ver = ver
			break
		    end
		end
	    end
	end
    end
end
if nvidia_pkg ~= nil then
    -- print("Try to install package " .. pkgname(nvidia_pkg) .. " .")
    markinstall(nvidia_pkg)
end

-- Find 32-bit packages for 64-bit system
if nvidia_pkg and os.getexecout("uname -m") == "x86_64" then
    if pkgfind("i586-libX11.32bit") then
	ver = pkgvercand(nvidia_pkg)
	if ver == nil then
	    ver = pkgvercur(nvidia_pkg)
	end
	for i, dep in ipairs(verdeplist(ver)) do
	    if string.find(dep.name, "nvidia_glx_", 0, true) == 1 then
		-- print(" require " .. dep.name)
		nvidia32_pkg = pkgfind("i586-" .. dep.name .. ".32bit")
		if nvidia32_pkg ~= nil then
		    markinstall(nvidia32_pkg)
		end
	    end
	end
    end
end

if nvidia_pkg == nil then
    print(_("NVIDIA driver not found for your kernel."))
    print(_("Be sure you did `apt-get update`."))
end
